package de.hdm_stuttgart.mi.exam;

import java.lang.annotation.Annotation;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Vector;

import org.junit.rules.TestWatcher;
import org.junit.runner.Description;

public class ResultCollector extends TestWatcher {

	public final Map<String, List<TestResult>> 
	failedByDescription = new HashMap<String,  List<TestResult>>(),
	successByDescription = new HashMap<String, List<TestResult>>();

	@Override
	protected void starting(Description description) {
		MarkingTest.currentTestClass = description;
	}

	@Override
	protected void failed(final Throwable e, final Description description) {
		putAppend(failedByDescription, description.getClassName(), new TestResult(description,
				getMarking(description), false));
	}

	@Override
	protected void succeeded(final Description description) {
		putAppend(successByDescription, description.getClassName(), new TestResult(description,
				getMarking(description), true));
	}

	private static void putAppend(final Map<String, List<TestResult>> map, final String key, final TestResult testresult) {
		List<TestResult> results = map.get(key);
		if (null == results) {
			results = new Vector<TestResult>();
			map.put(key, results);
		}
		results.add(testresult);
	}
	private static Marking getMarking(final Description description) {

		final Marking marking = description.getAnnotation(Marking.class);
		if (null == marking) { // Default values: 1 point
			return new Marking() {

				@Override
				public Class<? extends Annotation> annotationType() {
					return Marking.class;
				}

				@Override
				public int points() {
					return 1;
				}
			};

		}
		return marking;
	}

	public String currentToString() {
		
		final String currentTestClassName = MarkingTest.currentTestClass.getClassName();

		int reachablePoints = 0, reachedPoints = 0, numSuccessfulTests = 0;

		final List<TestResult> successfulTestResults = successByDescription.get(currentTestClassName);

		if (null != successfulTestResults) {
			for (final TestResult t : successfulTestResults) {
				reachablePoints += t.marking.points();
				reachedPoints += t.marking.points();
				numSuccessfulTests++;
			}
		}

		final StringBuffer failBuffer = new StringBuffer();

		final List<TestResult> failedTestResults = failedByDescription.get(currentTestClassName);

		if (null != failedTestResults) {
			for (final TestResult t : failedTestResults) {
				reachablePoints += t.marking.points();
				failBuffer.append("  -" + t.testMethodDisplayName + "\n");
			}
		}

		final StringBuffer buffer = new StringBuffer();

		buffer.append(currentTestClassName + ":\nYou have reached " + reachedPoints + " of "
				+ reachablePoints + " possible points in " + numSuccessfulTests + " successful tests. ");

		if (null != failedTestResults) {

			final int numFailed = failedTestResults.size();
			if (1 == numFailed) {
				buffer.append(" The following test still fails:\n");
			} else if (1 <  numFailed) {
				buffer.append("\n\nThe following " + numFailed + " tests still fail:\n");
			}
			buffer.append(failBuffer + "\n");
		}
		buffer.append("\n");
		return buffer.toString();
	}

	@Override
	public String toString() {

		int reachablePoints = 0, reachedPoints = 0, successfulTests = 0, failedTests = 0;

		for (final Entry<String, List<TestResult>> successEntries: successByDescription.entrySet()) {
			for (final TestResult t : successEntries.getValue()) {
				successfulTests++;
				reachablePoints += t.marking.points();
				reachedPoints += t.marking.points();
			}
		}

		final StringBuffer failBuffer = new StringBuffer();
		for (final Entry<String, List<TestResult>> failEntries: failedByDescription.entrySet()) {
			for (final TestResult t : failEntries.getValue()) {
				failedTests++;
				reachablePoints += t.marking.points();
				failBuffer.append("  -" + t.testMethodDisplayName + "\n");
			}
		}

		final StringBuffer buffer = new StringBuffer();

		buffer.append("You have reached " + reachedPoints + " of "
				+ reachablePoints + " possible points in "
				+ successfulTests + " successful tests\n");
		if (1 == failedTests) {
			buffer.append("\n\nThe following test still fails:\n");
		} else if (1 < failedTests) {
			buffer.append("\n\nThe following " + failedTests
					+ " tests still fail:\n");
		}
		buffer.append(failBuffer + "\n");
		return buffer.toString();
	}
}