package de.hdm_stuttgart.mi.sd1.aufgabe2;

import java.util.Arrays;

/**
 * <p>Verwaltung ganzzahliger Geldbeträge. Auf ein Konto können Zu- und
 *    Abbuchungen erfolgen.</p>
 *    
 * <p>Erfolgreiche Zu- und Abbuchungen werden intern protokolliert, um
 *    Entscheidungen über die Zulässigkeit von Abbuchungen treffen
 *    zu können.</p>
 *
 * <p>Hinweis: Die Methode void neueBuchung(final int betrag)
 *    implementiert bereits eine Protokollierung von Zu- und 
 *    Abbuchungen durch das Array 
 *    <code>zurueckliegendeBuchungen</code>.</p>
 *
 */
public class Konto {

   /**
    * Bis zu diesem Wert darf ein Konto überzogen werden.
    */
   static public final int DISPOLIMIT = 3000;

   int [] zurueckliegendeBuchungen = new int[0];

   int kontostand;
   
   /**
    * Neues Konto mit Anfangsbetrag 0.
    */
   public Konto() {
      kontostand = 0;
   }

   /**
    * <p>Neues Konto mit gegebenem Anfangsbetrag.</p>
    * 
    * <p>Ein Anfangsbetrag wird nicht als Buchung gewertet.</p>
    * 
    * @param kontostand Anfangsbetrag des neu erstellten Kontos.
    */
   public Konto(final int kontostand) {
      this.kontostand = kontostand;
   }

   /**
    * <p>Zubuchen eines Betrags zum Konto.</p>
    * 
    * @param betrag Der zuzubuchende Betrag muss positiv sein.
    * @return true, falls 0 &lt; betrag gilt, sonst false. In letzterem Fall
    *         erfolgt keine Änderung des Kontostands.
    */
   public boolean zubuchen(final int betrag) {
      
      if (0 < betrag) {
         neueBuchung(betrag);
         return true;
      } else {
         return false;
      }
   }

   /**
    * <p>Abbuchung eines Betrags vom Konto</p>
    * 
    * <p>Abbuchungen können genau dann erfolgen, wenn nachfolgend beschriebene Bedingungen gelten:</p>
    * 
    * <ol>
    *   <li>Der abzubuchende Betrag muss positiv sein.</li>
    *   <li>Durch die Abbuchung darf das Dispolimit von 3000 nicht unterschritten werden.</li>
    *   <li>Es sind maximal 5 aufeinanderfolgende erfolgreiche Abbuchungen möglich. Danach muss vor
    *       einer weiteren Abbuchung erst eine <b>erfolgreiche</b> Zubuchung erfolgen.</li>
    * </ol>
    * 
    * @param betrag <blockquote>Der abzubuchende Betrag muss positiv sein.</blockquote>
    * @return <blockquote>true, falls die genannten Bedingungen gelten, sonst false. In letzterem Fall
    *         erfolgt keine Änderung des Kontostands.</blockquote>
    */
   public boolean abbuchen(final int betrag) {
      if (0 < betrag &&
            -DISPOLIMIT <= kontostand - betrag &&
            (anzahlAbbuchungenInFolge() < 5) ) {
         neueBuchung(-betrag);
         return true;
      } else {
         return false;
      }
   }

   private int anzahlAbbuchungenInFolge() {
      
      int anzahlAbbuchungen = 0;
      for (int i = zurueckliegendeBuchungen.length - 1; 0 <= i; i--) {
         if (zurueckliegendeBuchungen[i] < 0) {
            anzahlAbbuchungen++;
         } else {
            return anzahlAbbuchungen;
         }
      }
      return anzahlAbbuchungen;
   }
   
   /**
    * Aktueller Kontostand.
    * @return <blockquote>Guthaben sind positiv, Schulden negativ.</blockquote>
    */
   public int getKontostand() {
      return kontostand;
   }
   
   void neueBuchung(final int betrag) {
         // Simpel, aber imperformant und verschwenderisch in Bezug
         // auf Speichernutzung.
         zurueckliegendeBuchungen = Arrays.copyOf(zurueckliegendeBuchungen, zurueckliegendeBuchungen.length + 1);
         zurueckliegendeBuchungen[zurueckliegendeBuchungen.length - 1] = betrag;
         kontostand += betrag;
   }
}
