package de.hdm_stuttgart.mi.sd1.test.aufgabe1;

import org.junit.Assert;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runners.MethodSorters;

import de.hdm_stuttgart.mi.exam.unitmarking.ExaminationTestDefaults;
import de.hdm_stuttgart.mi.exam.unitmarking.Marking;
import de.hdm_stuttgart.mi.sd1.aufgabe1.Gaspreise;

/**
 * Teste Jährliche Gaspreise basierend auf Verbrauchsangaben.
 *
 */
@SuppressWarnings({"static-method"})
@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class Test_Gaspreise extends ExaminationTestDefaults {

   /**
    * Toleranz bei Rundungsfehlern in double Vergleichen. Erläuterung
    * siehe z.B. {@link Assert#assertEquals(double, double, double)}.
    */
   static private final double testDelta = 1.E-10;


   /**
    * Geringverbrauchertarif, Nullverbrauch
    */
   @Test
   @Marking(points=2)
   public void test_010_geringverbrauchNullGasabnahme() {
      Assert.assertEquals(Gaspreise.KLEINVERBRAUCHER_GRUNDPREIS,
            Gaspreise.getKostenKleinverbrauch(0),
            testDelta);
   }

   /**
    * Geringverbrauchertarif, Verbrauch 1500 und 3000
    */
   @Test
   public void test_012_gerinverbrauchZweiWerte() {
      double erwartet = Gaspreise.KLEINVERBRAUCHER_GRUNDPREIS
                      + Gaspreise.KLEINVERBRAUCHER_ARBEITSPREIS * 1500;
      Assert.assertEquals(erwartet,
            Gaspreise.getKostenKleinverbrauch(1500),
            testDelta);

      erwartet = Gaspreise.KLEINVERBRAUCHER_GRUNDPREIS
            + Gaspreise.KLEINVERBRAUCHER_ARBEITSPREIS * 3000;
      Assert.assertEquals(erwartet,
            Gaspreise.getKostenKleinverbrauch(3000),
            testDelta);
   }

   /**
    * Großverbrauchertarif, Nullverbrauch
    */
   @Test
   public void test_020_grossverbrauchNullGasabnahme() {
      Assert.assertEquals(Gaspreise.GROSSVERBRAUCHER_GRUNDPREIS,
            Gaspreise.getKostenGrossverbrauch(0),
            testDelta);
   }


   /**
    * Großverbrauchertarif, Verbrauch 20000 und 60000
    */
   @Test
   public void test_022_grossverbrauchZweiWerte() {
      double erwartet = Gaspreise.GROSSVERBRAUCHER_GRUNDPREIS
            + Gaspreise.GROSSVERBRAUCHER_ARBEITSPREIS * 20000;
      Assert.assertEquals(erwartet,
            Gaspreise.getKostenGrossverbrauch(20000),
            testDelta);

      erwartet = Gaspreise.GROSSVERBRAUCHER_GRUNDPREIS
            + Gaspreise.GROSSVERBRAUCHER_ARBEITSPREIS * 60000;

      Assert.assertEquals(erwartet,
            Gaspreise.getKostenGrossverbrauch(60000),
            testDelta);
  }
  /**
   * Gleichheit beider Tarife bei 4375 Kilowattstunden Verbrauch.
   */
  @Test
  public void test_080_tarifeIdentisch() {
    Assert.assertEquals(Gaspreise.getKostenGrossverbrauch(4375),
        Gaspreise.getKostenKleinverbrauch(4375),
        testDelta);
  }

  /**
   * Günstigster Tarif bei 6000 und 2000 Kilowattstunden Verbrauch.
   */
  @Test
  public void test_100_guenstigsterTarif() {
    Assert.assertEquals(290,
        Gaspreise.getKostenGuenstigsterTarif(2000),
        testDelta);
    Assert.assertEquals(640,
        Gaspreise.getKostenGuenstigsterTarif(6000),
        testDelta);
  }
}