package de.hdm_stuttgart.mi.sd1.test.helper;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.Arrays;
import java.util.stream.Collectors;


public class ReflectClass {
   public ReflectClass(final String className) {
      this.className = className;
      Class<?> classTmp = null;
      String tmpMsg = null;
      try {
         classTmp = Class.forName(className);
      } catch (final ClassNotFoundException e) {
         tmpMsg = "Die Klasse '" + className + "' wurde nicht gefunden:";
      }
      classZ = classTmp;
      msg = tmpMsg;
   }
   public final String className;
   public final Class<?> classZ; 
   public final String msg;

   /**
    * @param arguments
    * @return Class names joined by ',', possibly stripped of "java.lang" component
    */
   public static String formatArguments(final Class<?>[] arguments) {

      return Arrays.stream(arguments).
            map(c -> normalizeClassname(c)).
            collect(Collectors.joining(", ")).
            toString();
   }

   /**
    * 
    * @param classz
    * @return The classname possible stripped of "java.lang" component
    */
   public static String normalizeClassname(final Class<?> classz) {
      final Package javaDotLang = Package.getPackage("java.lang"),
            hostingPackage = classz.getPackage();

      if (null == hostingPackage || !hostingPackage.equals(javaDotLang)) {
         return classz.getName();
      } else {
         return classz.getSimpleName();
      }
   }
      
   /**
    * 
    * @param classz
    * @return A string containing all non-private attribute names
    */
   public static String getNonPrivateAttributes(final Class<?> classz) {

      final Field[] fields = classz.getDeclaredFields();
      final StringBuffer results = new StringBuffer();
      boolean found = false;
      for (final Field f: fields) {
         if (0 == (Modifier.PRIVATE & f.getModifiers())) {
            if (found) {
               results.append(", ");
            }
            results.append(f.getName());
            found = true;
         }
      }
      if (found) {
         return results.toString();
      }
      return null;
   }
   /**
    * Get modifier cleartext name according to
    * https://docs.oracle.com/javase/8/docs/api/java/lang/reflect/Modifier.html#field.summary
    * 
    * @param modifier
    * @return Clear text name e.g. "final"
    */
   static public String getModifierString(final int modifier) {
      
      switch (modifier) {
      case Modifier.ABSTRACT: return "abstract";
      case Modifier.FINAL: return "final";
      case Modifier.INTERFACE: return "interface";
      case Modifier.NATIVE: return "native";
      case Modifier.PRIVATE: return "private";
      case Modifier.PROTECTED: return "protected";
      case Modifier.PUBLIC: return "public";
      case Modifier.STATIC: return "static";
      case Modifier.STRICT: return "strict";
      case Modifier.SYNCHRONIZED: return "synchronized";
      case Modifier.TRANSIENT: return "transient";
      case Modifier.VOLATILE: return "volatile";

      default:
         return "Unknown modifier value " + modifier;
      }
      
   }
   
}