package de.hdm_stuttgart.mi.sd1.aufgabe1;

import java.util.Arrays;

/**
 * Hilfsmethoden für Arrays.
 */
public class ArrayHelper {

  /**
   * <p>Enthält ein Array ganzzahliger Werte mindestens einen mehrfach auftretenden Wert? Beispiele:</p>
   *
   * <ul>
   * <li>In <code>{1, -5, 3, 1, 17, 6, 1}</code> ist der Wert 1 dreifach enthalten.</li>
   * <li><code>{-5, 5, 4, 9}</code> enthält keinen mehrfach auftretenden Wert.</li>
   * </ul>
   *
   * <p>Praktische Verwendung:</p>
   * <pre>final int[] werte = {2, 4, 3, 0, 1, 2};
   * final boolean result = ArrayHelper.enthaeltDuplikate(werte);</pre>
   *
   * <p>Wegen des doppelten Auftretens des Werts »2« hat <code>result</code> den Wert <code>true</code>.</p>
   *
   * @param werte null oder ein Array beliebiger Ganzzahlen
   * @return <code>true</code>, falls mindestens ein Wert doppelt vorkommt, ansonsten <code>false</code>.
   */
  static public boolean enthaeltDuplikate(final int[] werte) {

//    // Mit streams:
//    if (null == werte) {
//      return false;
//    } else {
//      return enthaeltDuplikate(Arrays.stream(werte).boxed().toArray());
//    }

    if (null != werte) {
      for (int i = 0; i < werte.length - 1; i++) {
        final int start = werte[i];
        for (int j = i + 1; j < werte.length; j++) {
          if (start == werte[j]) {
            return true;
          }
        }
      }
    }
    return false;
  }

  /**
   * <p>Analog zu {@link #enthaeltDuplikate(int[])}. </p>
   *
   * <p>Praktische Verwendung:</p>
   *
   * <pre>final String[] werte = {"One", null, "true", null};
   * final boolean result = ArrayHelper.enthaeltDuplikate(werte);</pre>
   *
   * <p>Da es keinen doppelt auftretenden String gibt und <code>null</code> Werte ignoriert werden, erhält
   * <code>result</code> den Wert <code>false</code>.</p>
   *
   * @param werte <code>null</code> oder ein Array aus Object und <code>null</code> Werten.
   * @return <code>true</code>, falls mindestens ein String doppelt vorkommt, ansonsten <code>false</code>,
   * wobei enthaltene <code>null</code> Werte in Bezug auf Duplikate ignoriert werden.
   */
  static public boolean enthaeltDuplikate(final Object[] werte) {
    if (null != werte) {
      for (int i = 0; i < werte.length - 1; i++) {
        final Object start = werte[i];
        if (null != start) {
          for (int j = i + 1; j < werte.length; j++) {
            if (start.equals(werte[j])) {
              return true;
            }
          }
        }
      }
    }
    return false;
  }
}
