package de.hdm_stuttgart.mi.sd1.intmax;

/**
 * Dealing with array values
 *
 */
public class Max {

   /**
    * An array may contain multiple values being equal to the maximum.
    * The sample {-4, 5, 2, 5, 4} contains two elements being equal to
    * the array's maximum value 5 at index positions 1 and 3. This method
    * retrieves the first index of an array element being equal to the
    * maximum. In the current example the desired index is 1 pointing
    * to the second array element.
    * 
    * @param values An array of arbitrary values.
    * 
    * @return The smallest index of an array member being equal to the maximum. 
    * <dl>
    *   <dt><b>Precondition:</b></dt>
    *   <dd>0 &lt; values.length</dd>
    *   
    *   <dt><b>Postcondition:</b></dt>
    *   <dd>0 &lt;= return value &lt; values.length</dd>
    * </dl>
    */
   public static int getFirstMaxIndex(final int[] values) {
      int
      firstMaxIndex = 0;                    // The index of the first array element being equal to the maximum value

      for (int i = 1; i < values.length; i++) {
         if (values[firstMaxIndex] < values[i]) {// Another value larger than the current maximum candidate
            firstMaxIndex = i;           // New index to possible first maximum value
         }
      }
      return firstMaxIndex;
   }

   /**
    * <p>An array may contain multiple values being equal to the maximum.
    * The sample {-4, 5, 2, 5, 4} contains two elements being equal to
    * the array's maximum value 5 at index positions 1 and 3
    * respectively. This method retrieves all index values of array
    * elements being equal to the maximum and returns
    * them as an array in ascending order.</p>
    * <p>In the current example the returned array equals {1, 3}
    * pointing to the second and fourth array elements.</p>
    * 
    * @param values An array of arbitrary values.
    * 
    * @return  
    * <ul>
    *  <li>An empty array if null == values or 0 == values.length</li>
    *  <li>All index values in ascending order of any array member being equal to the 
    *      maximum otherwise</li>
    * </ul>
    */
   public static int[] getMaxIndexValues(final int[] values) {
      if (null == values || values.length == 0) {
         return new int[0];
      } 

      int 
      numberOfMaxValues = 1,               // The number of array elements being equal to the maximum.
      firstMaxIndex = 0;                   // The index of the first array element being equal to the maximum value
      // found so far.

      for (int i = 1; i < values.length; i++) {
         if (values[firstMaxIndex] == values[i]) {      // Another value being identical to the maximum?
            numberOfMaxValues++;
         } else if (values[firstMaxIndex] < values[i]) {// Value greater current maximum candidate?
            firstMaxIndex = i;                  // Raise index to new candidate maximum value and
            numberOfMaxValues = 1;              // reset number of values found so far.
         }
      }
      // Now we do know:
      // 1. The first max value's index aka firstMaxIndex
      // 2. the number of array members being equal to the array's maximum value.
      // We can thus allocate an array of required size:
      //
      final int[] ascendingIndexList = new int[numberOfMaxValues];

      // .. and fill in the corresponding index values
      int ascendingArrayIndex = 0;
      // We already know the array's first element:
      ascendingIndexList[ascendingArrayIndex++] = firstMaxIndex;
      // Let's fill in the remaining index values
      //
      for (int i = firstMaxIndex + 1; i < values.length; i++) {
         if (values[firstMaxIndex] == values[i]) {
            ascendingIndexList[ascendingArrayIndex++] = i;
         }
      }
      return ascendingIndexList;
   }
}