package de.hdm_stuttgart.mi.sda1.exam.rdbms;

import java.io.IOException;
import java.sql.SQLException;

import org.jdom2.Element;
import org.jdom2.JDOMException;
import org.jdom2.input.SAXBuilder;

/**
 * Reading XML Data and inserting to relational systems. 
 *
 */
public class Xml2Rdbms {
   
   final DbHandler db = new DbHandler();
   
   final SAXBuilder builder = new SAXBuilder();
   
   int fromLanguageId, toLanguageId;
   
   /**
    * Read XML data for RDBMS transfer.
    * 
    * @param dictionaryFileName XML filename containing dictionary data,
    * @throws SQLException Database related error.
    * @throws JDOMException Parsing exception
    * @throws IOException File access exception.
    */
   public void read(final String dictionaryFileName) throws SQLException, JDOMException, IOException {
      db.connect();
      
      final Element dictionary = builder.build(dictionaryFileName).getRootElement();
     
      fromLanguageId = db.addLanguageIfMissing(dictionary.getAttributeValue("fromLanguage"));
      toLanguageId = db.addLanguageIfMissing(dictionary.getAttributeValue("toLanguage"));
      
      for (final Element entry: dictionary.getChildren("entry")) {
         handleEntry(entry);
      }
   }
   
   /**
    * 
    * @param args Unused
    * @throws IOException 
    * @throws JDOMException 
    * @throws SQLException 
    */
   public static void main(String[] args) throws JDOMException, IOException, SQLException {
      final Xml2Rdbms x2r = new Xml2Rdbms();
      x2r.read("Schema/sampledata.xml");
      
   }
   
   

    void handleEntry(final Element entry) throws SQLException {
       final int fromNounId = db.addNounIfMissing(entry.getChild("from").getText(), fromLanguageId);
       for (final Element to: entry.getChild("translations").getChildren("to")) {
          final int toNounId = db.addNounIfMissing(to.getText(), toLanguageId);
          db.addTranslation(fromNounId, toNounId);
       }
   }
}

