  package de.hdm_stuttgart.mi.sda1.impl;

  import org.jdom2.Element;
  import org.jdom2.output.Format;
  import org.jdom2.output.XMLOutputter;

  import java.io.IOException;
  import java.io.OutputStream;
  import java.sql.Connection;
  import java.sql.DriverManager;
  import java.sql.ResultSet;
  import java.sql.SQLException;

  abstract public class DbRead {

     final String sqlQuery;

    public final Element html;
    private final Element table;

    private final Connection conn;

    public DbRead(final String sqlQuery) {
      this.sqlQuery = sqlQuery;
      html = new Element("html");
      append(append(html, "head"), "title", "Results");
      table = append(append(html, "body"), "table");

      final Element headerLine = append(table, "tr");
      append(headerLine, "th", "Rank");
      append(headerLine, "th", "Name");
      append(headerLine, "th", "Time / seconds");

      Connection connTmp = null;
      try {
        connTmp = DriverManager.getConnection(
            "jdbc:mysql://localhost/hdm?serverTimezone=UTC", "hdmuser", "XYZ");
      } catch (SQLException e) {
        System.err.println("Unable to establish connection: " + e);
        System.exit(1);
      }
      conn = connTmp;
    }

    protected abstract void insertRecord(final ResultSet result, final Element row, final int rank)
        throws SQLException;

    public void readDatabase() {
      try {
        final ResultSet results = conn.createStatement().executeQuery(sqlQuery);
        int rank = 1;
        while(results.next()) {
          insertRecord(results, append(table, "tr"), rank++);
        }
      } catch (final SQLException e) {
        System.err.println("Database access error: " + e);
        System.exit(1);
      }
    }


    static protected Element append(final Element parent, final String tagname) {
      final Element child = new Element(tagname);
      parent.addContent(child);
      return child;
    }
    static  protected void append(final Element parent, final String tagname, final String text) {
      final Element child = new Element(tagname);
      child.addContent(text);
      parent.addContent(child);
    }

    public void toHtml(final OutputStream out) {

      final XMLOutputter printer =
          new XMLOutputter(Format.getPrettyFormat());
      try {
        printer.output(html, out);
      } catch (final IOException e) {
        System.err.println("Unable to write output: " + e);
      }
    }
  }