package de.hdm_stuttgart.mi.sd1.array;

import java.util.Arrays;

public class MyArray {

  /**
   * <p>Turn a list of values into a sorted set not containing any duplicates. Example:</p>
   *
   * <pre>{6, 1, 5, 1, 6, 1} to {1, 5, 6}</pre>
   *
   * <p>The array of input values is to be left untouched.</p>
   *
   * @param values Unsorted values possibly containing duplicates.
   * @return An ordered set of distinct values.
   */
  public static int[] distinct (final int[] values) {

    if (0 == values.length) {
      return values; // Special case: Empty array
    }

    // Step 1: Cloning incoming array
    final int[] sortedValues = Arrays.copyOf(values, values.length);

    // Step 2: Sort copied array.
    Arrays.sort(sortedValues);

    // Step 3: Create helper array containing distinct index positions
    final int[] uniqueValuesIndex = new int[values.length];
    int uniqueValueIndexTop = 0;
    uniqueValuesIndex[uniqueValueIndexTop++] = 0; // Include first array element unconditionally

    for (int i = 1; i < sortedValues.length; i++) {
      if (sortedValues[i - 1] != sortedValues[i]) { // current value different from predecessor?
        uniqueValuesIndex[uniqueValueIndexTop++] = i; // Add to set of distinct index values
      }
    }

    // Step 4: Create and copy to result array
    final int[] result = new int[uniqueValueIndexTop]; // We found uniqueValueIndexTop distinct values

    for (int i = 0; i < uniqueValueIndexTop; i++) { // Copy distinct values to result array
      result[i] = sortedValues[uniqueValuesIndex[i]];
    }
    return result;
  }
}