package de.hdm_stuttgart.mi.marking;



/**
 * 
 * 
 */
public class Marks {
  

    /**
     * Read marks from the command line.
     *
     * @param marksAsStrings User entered mark strings e.g. {"1", "4", ...}
     *
     */
    public static void main( String[] marksAsStrings ) {
        if (0 == marksAsStrings.length) {
            System.out.println("Usage: marks <mark1 mark2 ...>");
        } else {
            // Converting mark strings to integer values
            final int[] marks = new int[marksAsStrings.length];
            for (int i = 0; i < marksAsStrings.length; i++) {
                final String markString = marksAsStrings[i];
                try {
                    marks[i] = Integer.valueOf(markString);
                } catch (final NumberFormatException ex) {
                    System.err.println("Error: Unable to process '" + markString + "'");
                    System.exit(1);
                }
            }

            final int[] frequencies = Marks.getFrequencies(marks);

            System.out.println("Mark|frequency");
            System.out.println("----+---------");

            for (int lowestMark = frequencies[0], i = 1; i < frequencies.length; i++) {
                System.out.println("   " + (lowestMark + i - 1) + "|" + frequencies[i]);
            }
        }
    }
        
    /**
     * Gathering frequencies of integer values. Consider:
     * 
     * {-3, 4 , 2, -2, 4, -3, 2, 4}
     * 
     * The resulting frequencies are:
     * 
     * Value | frequency
     * ------+---------
     *    -3 | 2
     *    -2 | 1
     *    -1 | 0
     *     0 | 0
     *     1 | 0
     *     2 | 2
     *     3 | 0
     *     4 | 3
     *
     * The lowest value -3 will be stored at array position 0. All subsequent array position
     * will carry frequency values.
     *
     * Thus the resulting array will be {-3, 2, 1, 0, 0, 0, 2, 0, 3}
     * 
     * 
     * @param values
     *  A non-empty list possibly containing duplicates.
     * @return
     *  An array containing the lowest value followed by input values' frequencies. The array index 0 corresponds to
     *  the smallest input value(s).
     */
    static public int[] getFrequencies(final int[] values) {
      
      // Step 1: We need to find the minimum and maximum value
   
      // Starting from the "worst" assumption.
      //
      int maxValue = Integer.MIN_VALUE,
          minValue = Integer.MAX_VALUE;
      
      for (final int v: values) {
        if (maxValue < v) {
          maxValue = v;
        }
        if (v < minValue) {
          minValue = v;
        }
      }
      
      // Now we need an array of appropriate size to collect the desired frequencies 
      
      final int[] ret = new int[maxValue - minValue + 2];
      ret[0] = minValue; // Storing lowest (starting) value
      
      // Collect the values' frequencies
      for (final int v: values) {
        ret[v - minValue + 1]++;
      }
      return ret;
    }
    
}