package de.hdm_stuttgart.mi.sd1.htmlformat;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.List;

/**
 * Reading address data from text file data source.
 */
public class AddressDataHandler {

  /**
   * Available address records.
   */
  public final List<Address> addresses = new ArrayList<Address>();
  
  /**
   * Parsing text file address data having the following format:
   * 
   * <pre>"firstName","lastName","companyName","address","city","county","postal","phone1","phone2","email","web"</pre>
   * 
   * The first line is expected to contain the above description and will be ignored.
   * 
   * @param filename Address data source.
   * 
   * @throws IOException Input file cannot be read
   * @throws FileNotFoundException File does not exist
   * @throws AddressParseError Input data file parse error
   * 
   */
  public AddressDataHandler(final String filename) 
      throws FileNotFoundException, IOException, AddressParseError {
    
    final BufferedReader reader = new BufferedReader(new FileReader(filename));
    int lineNumber = 0; // Needed to supply file positions for error messages
    for (String lineContent = reader.readLine(); lineContent != null; lineContent = reader.readLine()) {
      lineNumber ++;
      addresses.add(new Address(lineContent, lineNumber));
    }
    reader.close(); // Freeing OS resources no longer being needed. 
  }
  
  /**
   * Format the set of address records and write the result
   * to a stream.
   * @param out Desired output stream
   * @param formatter Formatting instance (may e.g. be text, HTML,...)
   */
  public void printAddresses(final PrintStream out, final AddressFormatter formatter) {
    formatter.printHead(out);
    
    for (final Address address: addresses) {
      formatter.printRecord(address, out);
    }
    
    formatter.printTail(out);
  }
}
