package de.hdm_stuttgart.mi.sd1.main;

/**
 * Calculate prime numbers in a yet inefficient way
 *
 */
public class PrimeNumbers {
  /**
   * @param args unused
   */
  public static void main(String[] args) {

    final long startTime = System.currentTimeMillis();
    final long numberOfPrimes = findPrimeNumbers(10_000_000);
    final long endTime = System.currentTimeMillis();
    
    System.out.println("Found " + numberOfPrimes + " prime numbers within " +
          (endTime - startTime) / 1000. + " seconds.");
  }
  
  /**
   * Find all prime numbers up to a given limit.
   * 
   * @param limit Test all integers from [2, limit]
   * whether they are prime numbers or not.
   * @return The number of primes found within [2, limit].
   * <dl>
      <dt>Precondition:</dt>
      <dd>2 &lt;= limit</dd>
    </dl>
   */
  public static long findPrimeNumbers(long limit) {
    long numPrimes = 0; 
    for (int i = 2; i <= limit; i++) {
      if (isPrime(i)) {
        numPrimes++;
      }
    }
    return numPrimes;
  }
  
  /**
   * Test, whether a given number is prime. 
   * @param candidate The number to be assessed
   * @return true if candidate is prime, false otherwise
   * <dl>
      <dt>Precondition:</dt>
      <dd>2 &lt;= candidate</dd>
    </dl>
   */
  public static boolean isPrime(final long candidate) {
    for (long i = 2; i * i <= candidate; i++) { // Just test up to square
      if (0 == candidate % i) {                 // root of candidate.
        return false;
      }
    }
    return true;
  }
}
