package de.hdm_stuttgart.mi.xmlstat;

import java.io.IOException;
import java.io.PrintStream;
import java.util.Map.Entry;

import org.jdom2.Comment;
import org.jdom2.DocType;
import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.Namespace;
import org.jdom2.output.Format;
import org.jdom2.output.XMLOutputter;

/**
 * Represent a XHTML DOM for output generation.
 */
public class HtmlTree {
   
   /**
    * 
    */
   static public final Namespace HTML = 
         Namespace.getNamespace("http://www.w3.org/1999/xhtml");
   private final Element tableBody = new Element("tbody", HTML).
         addContent(new Comment(" End of static, beginning of dynamic section"));



   /**
    * Create a HTML skeleton (&lt;html&gt;&lt;head&gt;...&lt;body&gt; ...
    * &lt;/body&gt;&lt;/head&gt;
    * to be filled in later.
    *
    * @param titleText 
    *   The document's title, e.g. "Available articles"
    *   
    */
   @SuppressWarnings("unused")
   public HtmlTree(final String titleText) {

      // We create a HTML skeleton including a yet empty table
      final Element 
         table = new Element("table", HTML).
            setAttribute("border", "1").
            setAttribute("style", "border-collapse: collapse;").
            addContent(tableBody),
         head = new Element("head", HTML).addContent(new Element("title", HTML).
               addContent(titleText)),
         body = new Element("body", HTML).addContent(new Element("h1", HTML).
               addContent(titleText)).addContent(table);
      

      final DocType doctype =  new DocType("html",
            "-//W3C//DTD XHTML 1.0 Strict//EN", 
            "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd");

      final Element htmlRoot = 
            new Element("html", HTML).
            addContent(new Comment(" Static content section")).
            addContent(head).addContent(body);
      
      new Document (htmlRoot, doctype);
   }
   
   /**
    * 
    * @param entry namespace among with corresponding elements.
    */

   public void appendNamespace(final  Entry<Namespace, ElementFrequency> entry) {

      final Namespace nSpace = entry.getKey();
      final String nSpacePrefix = nSpace.getPrefix();
      final String nSpaceContent;
      
      if (null != nSpacePrefix && !nSpacePrefix.equals("")) {
         nSpaceContent = nSpace.getURI() + ", prefix = " + nSpacePrefix;
      } else {
         nSpaceContent = nSpace.getURI();
      }

      tableBody.
      addContent(new Element("tr", HTML).
            addContent(new Element("td", HTML).setAttribute("colspan", "2").
                  addContent(nSpaceContent)));

      tableBody.
      addContent(new Element("tr", HTML).
            addContent(new Element("th", HTML).addContent("Element")).
            addContent(new Element("th", HTML).addContent("Frequency")));


      entry.getValue().entrySet().
      stream().
      sorted((e1, e2) -> e1.getKey().compareTo(e2.getKey())).// Sort by element name
      forEach(this::appendElement);
      
   }
   
   void appendElement(final Entry<String, Integer> entry) {
      final Element tr = new Element("tr", HTML).
            addContent(new Element("td", HTML).
                  addContent(entry.getKey())).
            addContent(new Element("td", HTML).
                  addContent(entry.getValue().toString()));
      tableBody.addContent(tr);
      
   }
   
   /**
    * Serialize the HTML DOM to a stream
    * @param out Output destination.
    */
   public void serialize(final PrintStream out){

      // Set formatting for the XML output
      final Format outFormat = Format.getPrettyFormat();

      // Serialize to console
      final XMLOutputter printer = new XMLOutputter(outFormat);
      try {
         printer.output(tableBody.getDocument(), System.out);
      } catch (IOException e) {
         e.printStackTrace();
         System.exit(1);
      }
   }
   /**
    * @return the table's &lt;tbody&gt; element
    */
   public Element getTable() {
      return tableBody;
   }
}