package de.hdm_stuttgart.mi.company.model;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.PostRemove;
import javax.persistence.PreRemove;
import javax.persistence.Table;
import javax.persistence.UniqueConstraint;

@Entity
@Table(
		uniqueConstraints= {@UniqueConstraint(columnNames={"project", "employee"})}
)
@SuppressWarnings("javadoc")
public class WorksFor {
	
	protected WorksFor (){}
	
	public WorksFor(final Project project, final Employee employee, final int weekly_hours) {
		setProject(project);
		setEmployee(employee);
		setWeekly_hours(weekly_hours);
	}

	@Id
	int id;
	
	@ManyToOne(cascade=CascadeType.PERSIST)
	@JoinColumn(name="project")
	Project project;

	@ManyToOne(cascade=CascadeType.PERSIST)
	@JoinColumn(name="employee")
	Employee employee;
	
	int weekly_hours;

	public Project getProject() {
		return project;
	}

	public void setProject(Project project) {
	   if (null == this.project) {
	      this.project = project;
	      project.employees.add(this);
	   } else if (this.project != project) {
         this.project.employees.remove(this);
         this.project = project;
         project.employees.add(this);
	   }
	}

	public Employee getEmployee() {
		return employee;
	}

	public void setEmployee(Employee employee) {
		if (null != this.employee && this.employee != employee) {
			this.employee.projects.remove(this);
		}
		this.employee = employee;
		employee.projects.add(this);
	}

	public int getWeekly_hours() {
		return weekly_hours;
	}

	public void setWeekly_hours(int weekly_hours) {
		this.weekly_hours = weekly_hours;
	}
	
	@PreRemove
	void preRemove() {
	   project.getEmployees().remove(this);
	   employee.getProjects().remove(this);
	}
	@PostRemove
	void postRemove() {
	   project = null;
	   employee = null;
	   weekly_hours = 0;
	}
	
}
