package de.hdm_stuttgart.mi.sda1.Db;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.ResultSet;

import java.util.ResourceBundle;

public class WriteRelationalData implements AutoCloseable {

  static public final ResourceBundle jdbcProperties = ResourceBundle.getBundle("jdbc");

  private final Connection conn;
  private final PreparedStatement insertLecturer,insertCourse;
  private final Statement eraseData;

  public WriteRelationalData(final Connection conn) throws SQLException {
    this.conn = conn;
    insertLecturer = conn.prepareStatement(
      "INSERT INTO Lecturer (ident, name, email) VALUES (?, ?, ?)",
      Statement.RETURN_GENERATED_KEYS);

    insertCourse = conn.prepareStatement(
      "INSERT INTO Course (number, lecturerer, title, description) VALUES (?, ?, ?, ?)",
      Statement.RETURN_GENERATED_KEYS);

    eraseData = conn.createStatement();
  }

  @Override
  public void close() throws Exception {
    conn.close();
  }

  public void eraseData() throws SQLException {
    eraseData.executeUpdate( ("DELETE FROM Course"));
    eraseData.executeUpdate( ("DELETE FROM Lecturer"));
  }
  public long insertLecturer(final String ident, final String name, final String email) throws SQLException {

    insertLecturer.setString(1, ident);
    insertLecturer.setString(2, name);
    insertLecturer.setString(3, email);

    if (1 == insertLecturer.executeUpdate()) {
      try (final ResultSet generatedKeys = insertLecturer.getGeneratedKeys()) {
        if (generatedKeys.next()) {
          return generatedKeys.getLong(1);
        }
      }
    }
    return -1;
  }
  public int insertCourse(final int number, final long lecturer, final String title, final String description)
    throws SQLException {
    insertCourse.setInt(1, number);
    insertCourse.setLong(2,   lecturer);
    insertCourse.setString(3, title);
    insertCourse.setString(4, description);

    return insertCourse.executeUpdate();
  }
}