package de.hdm_stuttgart.mi.sda1.nocast;

import java.io.IOException;
import java.util.List;

import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.JDOMException;
import org.jdom2.Namespace;
import org.jdom2.filter.ElementFilter;
import org.jdom2.input.SAXBuilder;
import org.jdom2.xpath.XPathExpression;
import org.jdom2.xpath.XPathFactory;

/**
 * Searching for images {@code <img src="...">}
 * in XHTML documents.
 */
public class DomXpath {
  private final SAXBuilder builder = new SAXBuilder();

  static final Namespace htmlNamespace = Namespace.getNamespace("html", "http://www.w3.org/1999/xhtml");

  static final XPathExpression<Element> xpathSearchImg =
          XPathFactory.instance().compile(
                  "//html:img",    // Cf. namespace and prefix definition in htmlNamespace.
                  new ElementFilter(),
                  null,         // Cf. http://www.jdom.org/pipermail/jdom-interest/2012-May/016850.html
                  htmlNamespace);

  /**
   * Read an XHTML document and search for {@code <img src="...">} elements. Corresponding
   * image filenames will be written to standard output.
   *
   * Precondition: The document's root is expected belonging to the HTML namespace,
   * i.e. {@code <html xmlns="http://www.w3.org/1999/xhtml">}.
   *
   * @param xhtmlFilename The XHTML's filename to be examined.
   * @return List of images
   * @throws JDOMException General parsing fault
   * @throws IOException   file access problem
   */
  public List<Element> process(final String xhtmlFilename)
    throws JDOMException, IOException {

    final Document htmlInput = builder.build(xhtmlFilename);
    return xpathSearchImg.evaluate(htmlInput);
  }
}