package de.hdm_stuttgart.mi.javastreams;

import static org.hamcrest.MatcherAssert.assertThat;

import java.util.*;
import java.util.stream.Collector;
import java.util.stream.Collectors;

import com.github.npathai.hamcrestopt.OptionalMatchers;
import org.hamcrest.Matchers;
import org.junit.Assert;
import org.junit.FixMethodOrder;
import org.junit.Test;

import com.google.common.collect.ImmutableList;
import com.google.common.collect.ImmutableMap;

import de.hdm_stuttgart.mi.javastreams.Student.Sex;
import org.junit.runners.MethodSorters;

/**
 * Testing functional queries.
 */
@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class Java8FunctionalTest {

   static private final List<Student> students = ImmutableList.of(
       new Student("Fred",   2, Student.Sex.MALE,   "fred@example.com")
       ,new Student("Jane",  1, Student.Sex.FEMALE, "jane@kiv.de")
       ,new Student("George",4, Student.Sex.MALE,   "george@math.edu")
       ,new Student("Bob",   2, Student.Sex.MALE,   "bob@uk.edu")
       ,new Student("Kim",   2, Student.Sex.FEMALE, "wilde@serious.de")
   );

   /**
    * <p>Order all male students by email and create a {@code List<String>} of their respective
    * names alphabetically ordered eliminating possible duplicates. Implementation hints:</p>
    *
    *  <pre
    *  >{"Fred",   2, Student.Sex.MALE,   "fred@example.com"}
    *   {"Jane",   1, Student.Sex.FEMALE, "jane@kiv.de"}
    *   {"George", 4, Student.Sex.MALE,   "george@math.edu"}
    *   {"Bob",    2, Student.Sex.MALE,   "bob@uk.edu"}
    *   {"Kim",    2, Student.Sex.FEMALE, "wilde@serious.de"}</pre>
    *
    *  <p>Result: {"Bob", "Fred", "George"}</p>
    */
   @Test
   public void allMaleDistinctNameOrderedByEmail() {

      final List<String> emails =
          students.parallelStream().
              filter(s -> s.gender == Sex.MALE).
              sorted(Comparator.comparing(Student::getEmailAddress)).
              map(Student::getName).
              distinct().
              collect(Collectors.toList());

      assertThat(
          emails,
          Matchers.equalTo(
              ImmutableList.of("Bob", "Fred", "George")
          )
      );
   }

   /**
    * <p>Summing up all students' marks having a German email address.</p>
    *
    * <dl>
    *
    *     <dt>List of all students having a german E-mail:</dt>
    *     <dd>
    * <ul>
    *     <li><code>{"Jane", 1, Student.Sex.FEMALE, "jane@kiv.de"}</code></li>
    *     <li><code>{"Kim",  2, Student.Sex.FEMALE, "wilde@serious.de"}</code></li>
    * </ul>
    *     </dd>
    *
    *     <dt>Summing up related marks:</dt>
    *     <dd>Expected value: 1 + 2 == 3</dd>
    * </dl>
    *
    * <p>Implementation hint: Process a stream of {@link Student} instances by:</p>
    *
    * <ol>
    *     <li>Filter students having a german email.</li>
    *     <li>Map objects to mark values of type int.</li>
    *     <li>Provide a suitable
    *     <a href="https://docs.oracle.com/javase/tutorial/collections/streams/reduction.html#reduce"
    *     >reduce operation</a>. You may safely assume the existence of at least one student.</li>
    * </ol>
    *
    * <p>You may also want to read
    *  <a href="https://docs.oracle.com/javase/tutorial/collections/streams/reduction.html#reduce"
    *  >https://docs.oracle.com/javase/tutorial/collections/streams/reduction.html#reduce</a>.</p>
    */
   @Test
   public void markSumAllStudentsMarksHavingEmail_Dot_de() {

      final int sumMarksOfStudentsHavingGermanEmail = 123; /* TODO: Correct me!*/

      assertThat(sumMarksOfStudentsHavingGermanEmail, Matchers.equalTo(3));
   }

   /**
    * <p>A comma separated string containing all students' alphabetically ordered email addresses:</p>
    *
    * <p>{"bob@uk.edu, fred@example.com, george@math.edu, ..."}</p>
    *
    * <p>Implementation hint: Process a stream of {@link Student} instances by:</p>
    *
    * <ol>
    *   <li>Map {@link Student} instances to their emails.</li>
    *   <li>Sort alphabetically.</li>
    *   <li>Use <a href="https://www.baeldung.com/java-8-collectors">a suitable</a> {@link Collector}.</li>
    * </ol>
    */
   @Test
   public void orderedCharacterSeparatedEmailList() {

      final String csvSortedEmails = "weirdo!"; /* TODO: Correct me!*/

      assertThat(
          csvSortedEmails,
          Matchers.equalTo("bob@uk.edu, fred@example.com, george@math.edu, jane@kiv.de, wilde@serious.de")
      );
   }

   /**
    * Achieves the same result as in {@link #orderedCharacterSeparatedEmailList()}
    * internally using <a href="https://www.baeldung.com/java-8-streams#reduction">reduction</a>:
    *
    * <p>Implementation hint: Process a stream of {@link Student} instances by:</p>
    *
    * <ol>
    *   <li>Map {@link Student} instances to their emails.</li>
    *   <li>Sort alphabetically.</li>
    *   <li>Implement a suitable reducing operation turning a stream of email adresses into
    *       a single String containing comma separated email adresses.</li>
    *   <li>You may safely assume at least one email to be present.</li>
    * </ol>
    */
   @Test
   public void orderedCharacterSeparatedEmailList2() {

      final Optional<String> emailCsvList = Optional.of("Weirdo!"); /* TODO: Correct me!*/

      assertThat(emailCsvList, OptionalMatchers.isPresentAndIs(
          "bob@uk.edu, fred@example.com, george@math.edu, jane@kiv.de, wilde@serious.de"));
   }

   /**
    * <p>Get the average mark of all female students as double value.</p>
    *
    *  <pre
    *  >{"Jane", 1, Student.Sex.FEMALE, "jane@kiv.de"}
    *   {"Kim",  2, Student.Sex.FEMALE, "wilde@serious.de"}</pre>
    *
    *
    * <p>Female student marks sum: 1 + 2 == 3</p>
    *
    * <p>Result: average = 3. / 2 == 1.5</p>
    *
    */
   @Test
   public void averageMarkFemaleStudents() {

      final OptionalDouble femaleAverage = OptionalDouble.of(23.43); /* TODO: Correct me!*/

      // No suitable matcher on offer
      Assert.assertTrue(femaleAverage.isPresent());
      Assert.assertEquals((1 + 2) / 2., femaleAverage.getAsDouble(), 1.E-20);

   }

   /**
    * Create a {@code Map<Student.Sex, List<String>>} grouping students by sex and collecting their respective names:
    *
    * {
    *  Female: {"Jane", "Kim"},
    *  Male: {"Fred", "George", "Bob"}
    * }
    *
    */
   @Test
   public void studentNamesBySex() {

      final Map<Student.Sex, List<String>> studentnamesBySex = new HashMap<>(); /* TODO: Correct me!*/

      assertThat(
          studentnamesBySex,
          Matchers.equalTo(
              ImmutableMap.of (
                  Student.Sex.MALE,   ImmutableList.of("Fred", "George", "Bob"),
                  Student.Sex.FEMALE, ImmutableList.of("Jane", "Kim")
              )
          )
      );
   }

   /**
    * Marking frequencies:
    *
    * Mark 1: "Jane"               --- 1 student
    * Mark 2: "Fred", "Bob", "Kim" --- 3 students
    * Mark 4: "George"             --- 1 student
    *
    * Result:
    *
    * <pre>{
    *  1: 1,
    *  2: 3,
    *  4: 1
    * }</pre>
    *
    * <p>This one is hard!</p>
    *
    */
   @Test
   public void markingFrequencies() {
      final Map<Integer, Integer> frequencyByMark =new HashMap<>(); /* TODO: Correct me!*/

      assertThat(
          frequencyByMark,
          Matchers.equalTo(
              ImmutableMap.of(
                  1,   1,
                  2,   3,
                  4,   1
              )
          )
      );
   }

   /**
    * Students' names as {@code List<String>} in ascending order grouped by by mark values:
    * <pre>{
    *   1: {"Jane"}
    *   2: {"Bob", "Fred", "Kim"}
    *   4: {"George"}
    * }</pre>
    *
    *
    * <p>This one is even harder!</p>
    */
   @Test
   public void studentnamesByMark() {

      final Map<Integer, List<String>> namesByMark =new HashMap<>(); /* TODO: Correct me!*/

      assertThat(
          namesByMark,
          Matchers.equalTo (
              ImmutableMap.of(
                  1,   ImmutableList.of("Jane"),
                  2,   ImmutableList.of("Bob", "Fred", "Kim"),
                  4,   ImmutableList.of("George")
              )
          )
      );
   }
}