package de.hdm_stuttgart.mi.sda1.catalog2sql;

import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

import de.hdm_stuttgart.mi.sda1.catalog2sql.handler.CatalogContentHandler;
import de.hdm_stuttgart.mi.sda1.saxerrorhandler.handler.SaxErrorHandler;

/**
 * A simple SAX parser demo
 * 
 */
public class Xml2Rdbms {
   private static final Logger log = LogManager.getLogger(Xml2Rdbms.class);

   /**
    * @param args Unused
    * @throws SAXException
    * @throws ParserConfigurationException
    * @throws IOException
    */
   public static void main(String[] args) throws ParserConfigurationException,
         SAXException, IOException {

      log.info(Messages.getString("Xml2Rdbms.0"));
      final SAXParserFactory saxPf = SAXParserFactory.newInstance();
      final SAXParser saxParser = saxPf.newSAXParser();
      final XMLReader xmlReader = saxParser.getXMLReader();

      
      // Register Driver
      final String sqlDriverClassName = Messages.getString("Xml2Rdbms.jdbcDriverName"); 
      try {
         Class.forName(sqlDriverClassName);
      } catch (ClassNotFoundException e) {
         Helper.exitWithErrorMessage(Messages.getString("Xml2Rdbms.errMsgUnableRegisterDriverClass") + sqlDriverClassName + Messages.getString("Xml2Rdbms.3"), 1);
      }


      // Opening a JDBC connection
      // 
      Connection conn = null;
      
      final String jdbcConnectionUrl = Messages.getString("Xml2Rdbms.jdbcUrl"); 
      final String userName = Messages.getString("Xml2Rdbms.jdbcUser"); 
      try {
         conn = DriverManager.getConnection(jdbcConnectionUrl, userName, Messages.getString("Xml2Rdbms.jdbcPasswd")); 
      } catch (SQLException e) {
         Helper.exitWithErrorMessage(Messages.getString("Xml2Rdbms.errMsgUnableToConnect") + userName + Messages.getString("Xml2Rdbms.8") + 
               jdbcConnectionUrl + Messages.getString("Xml2Rdbms.9"), 1); 
      }
      
      DataInsert dataInsert = null;
      try {
         dataInsert = new DataInsert(conn);
      } catch (SQLException e) {
         Helper.exitWithErrorMessage(Messages.getString("Xml2Rdbms.errMsgUnableInitDataInsert"), 1); 
      }

      log.info(Messages.getString("Xml2Rdbms.11"));
      xmlReader.setContentHandler(new CatalogContentHandler(dataInsert));
      xmlReader.setErrorHandler(new SaxErrorHandler(System.err));

      final String xmlDocumentInstanceFilename = Messages.getString("Xml2Rdbms.xmlSourceFileName");
      log.info(Messages.getString("Xml2Rdbms.13") + xmlDocumentInstanceFilename + Messages.getString("Xml2Rdbms.14")); 
      xmlReader.parse(xmlDocumentInstanceFilename);

      // Closing
      try {
         conn.close();        // Closing Connection
      } catch (SQLException e) {
         Helper.exitWithErrorMessage(Messages.getString("Xml2Rdbms.errMsgUnableCloseRdbms"), 1); 
      }
   }
   
}
