package de.hdm_stuttgart.mi.sda1.sql2catalog;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import org.jdom2.Element;

/**
 * Reading product data
 *
 */
public class RdbmsAccess {
	
	final PreparedStatement selectProduct, selectDescription;

	/**
	 * @param conn Destination RDBMS
	 * @throws SQLException Error on preparing SQL statements for later access.
	 */
	public RdbmsAccess(final Connection conn) throws SQLException {
	   conn.setAutoCommit(false);
	   selectProduct = conn.prepareStatement("SELECT * FROM Product");
	   selectDescription = conn.prepareStatement(
	         "SELECT * FROM Description WHERE product = ? ORDER BY orderIndex");
	}
	
	/**
	 * Appending a catalog entry.
	 * @param root Sink for appending element.
	 * @throws SQLException Database related problem.
	 */
	public void appendProducts(final Element root) throws SQLException {
	   
	   try (final ResultSet products = selectProduct.executeQuery()) {
	      while (products.next()) {
	         final int productId = products.getInt("id");
	         final Element productElement = new Element("product");
	         root.addContent(productElement);
	         productElement.setAttribute("id", "" + productId);
	         
	         final Element nameElement = new Element("name");
	         productElement.addContent(nameElement);
	         nameElement.addContent(products.getString("name"));
	         
	         addDescriptions(productId, productElement);

	         final int productAge = products.getInt("age");
	         if (!products.wasNull()) {
	            final Element ageElement = new Element("age");
	            productElement.addContent(ageElement);
	            ageElement.addContent("" + productAge);
	         }
	      }
	   }
	}
	
	private void addDescriptions(final int productId, final Element productElement) throws SQLException {
	   selectDescription.setInt(1, productId);
	   
	   try (final ResultSet descriptions = selectDescription.executeQuery()) {
	      while (descriptions.next()) {
	         final Element descriptionElement = new Element("description");
	         productElement.addContent(descriptionElement);
	         descriptionElement.addContent(descriptions.getString("text"));
	      }
	   }
	}
}
