package com.restfully.shop.services;

import java.io.InputStream;
import java.net.URI;
import java.util.Collection;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicInteger;

import javax.ws.rs.Consumes;
import javax.ws.rs.DELETE;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Response;

import org.jboss.resteasy.annotations.providers.jaxb.XmlHeader;

import com.restfully.shop.domain.Customer;
import com.restfully.shop.helper.PrettyPrint;

@SuppressWarnings("javadoc")
@Path("/customers")
public class CustomerResource {
   
   public final static String xsltUrl = "<?xml-stylesheet type='text/xsl' href='/static/customer2html.xsl' ?>";

   private Map<Integer, Customer> customerDB = new ConcurrentHashMap<Integer, Customer>();
   private AtomicInteger idCounter = new AtomicInteger();

   public CustomerResource() {}

   @GET
   @Produces("application/xml")
   @PrettyPrint
   @XmlHeader(xsltUrl)
   public Collection<Customer> getAllCustomers() {
      return customerDB.values();
   }
   
   @POST
   @Consumes("application/xml")
   public Response createCustomer(final Customer customer) {
      
      customer.setId(idCounter.incrementAndGet());
      customerDB.put(customer.getId(), customer);
      System.out.println("Created customer " + customer.getId() + ", first name =" + customer.getFirstName() + ", city=" + customer.getCity() );
      return Response.created(URI.create("/customers/" + customer.getId())).build();
   }

   @GET
   @Path("{id}")
   @Produces("application/xml")
   @PrettyPrint
   @XmlHeader(xsltUrl)
   public Customer getCustomer(@PathParam("id") int id) {
      final Customer customer = customerDB.get(id);
      if (customer == null) {
         throw new WebApplicationException(Response.Status.NOT_FOUND);
      } else {
         return customer;
      }
   }

   @PUT
   @Path("{id}")
   @Consumes("application/xml")
   public void updateCustomer(@PathParam("id") int id, Customer customer) {
      Customer current = customerDB.get(id);
      if (current == null) {
         throw new WebApplicationException(Response.Status.NOT_FOUND);
      } else {
      customer.setId(current.getId());
      customerDB.put(id, customer);
      }
   }

   @DELETE
   @Path("{id}")
   public void deleteCustomer(@PathParam("id") int id, InputStream is) {
      final Customer removed = customerDB.remove(id);
      if (null == removed) {
         throw new WebApplicationException(Response.Status.NOT_FOUND);
      }
   }
}
