package de.hdm_stuttgart.mi;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.SQLException;

import com.vaadin.data.Property.ValueChangeEvent;
import com.vaadin.data.Property.ValueChangeListener;
import com.vaadin.data.fieldgroup.FieldGroup.CommitException;
import com.vaadin.navigator.Navigator;
import com.vaadin.navigator.View;
import com.vaadin.navigator.ViewChangeListener.ViewChangeEvent;
import com.vaadin.ui.Button;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Button.ClickListener;
import com.vaadin.ui.CustomComponent;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.Notification;
import com.vaadin.ui.PasswordField;
import com.vaadin.ui.VerticalLayout;

import de.hdm_stuttgart.mi.entities.User;
import de.hdm_stuttgart.mi.tools.MyBeanFieldGroup;
import de.hdm_stuttgart.mi.tools.ValidateGlobal;
import de.hdm_stuttgart.mi.tools.ValidateSecondIdenticalInput;


/**
 * @author goik
 *
 */
public class Register extends CustomComponent implements View, ValidateGlobal {
	
	private static final long serialVersionUID = -1309958813377546099L;
	/**
	 * This view's unique identifier.
	 */
	public final static String viewId = Register.class.getCanonicalName();

	private final static String
	  jdbcDriverClass = "com.mysql.jdbc.Driver",
	  jdbcUrl="jdbc:mysql://localhost:3306/hdm?useSSL=false",
	  jdbcUser="hdmuser",
	  jdbcPassword="XYZ";
	
	static { 
		try {
			Class.forName(jdbcDriverClass);
			System.out.println("JDBC driver initialization completed");
		} catch (ClassNotFoundException e) {
			System.err.println("Unable to find JDBC driver");
		}
	}

	transient Connection dbConnection;
	transient PreparedStatement insertUserStatement = null; 
	
	final Button
			cancelButton = new Button("Cancel"),
			okButton = new Button("OK");
	
	// The backing Pojo
	final User user = new User();
	
	// The Pojo's editor form
	final MyBeanFieldGroup<User> userForm = new MyBeanFieldGroup<User> (User.class, this);

	// Password has to be entered twice for consistency check
	final PasswordField password1Field, password2Field = new PasswordField ("Repeat Pw.");

	/**
	 * Create a registration instance allowing to navigate to
	 * other views.
	 * 
	 * @param navigator
	 */
	public Register(Navigator navigator) {
		
		try {

			dbConnection = DriverManager.getConnection(jdbcUrl, jdbcUser, jdbcPassword);
			insertUserStatement = dbConnection.prepareStatement(
					"INSERT INTO User (email, password, cname) VALUES (?, ?, ?)");
		} catch (SQLException e1) {
			System.err.println("Unable to initialize Database");
			e1.printStackTrace();
		}
		
		final VerticalLayout mainLayout = new VerticalLayout();
		setCompositionRoot(mainLayout);
		mainLayout.addComponents(userForm.buildAndBind("Name", "name"));
		mainLayout.addComponents(userForm.buildAndBind("Email", "email"));
		
		password1Field = // We need to compare both password entries for equality later
				userForm.buildAndBind("Password", "password", PasswordField.class);
		mainLayout.addComponents(password1Field);
		
		password2Field.addValidator(new ValidateSecondIdenticalInput(password1Field));
		password1Field.addValidator(new ValidateSecondIdenticalInput(password2Field));
		mainLayout.addComponents(password2Field);

		{// Preparing cancel/OK buttons
			final HorizontalLayout buttonLayout = new HorizontalLayout();
			buttonLayout.addComponent(cancelButton);
			buttonLayout.addComponent(okButton);
			okButton.setEnabled(false); // Wait until sensible data has been entered
			
			mainLayout.addComponent(buttonLayout);
		}
		cancelButton.addClickListener(new ClickListener() {
			private static final long serialVersionUID = -2794080599262943594L;
			@Override public void buttonClick(ClickEvent event) {navigator.navigateTo(MyLogin.viewId);}
		});

		okButton.addClickListener(new ClickListener() { 
			private static final long serialVersionUID = 3427398691509635900L;

			@Override public void buttonClick(ClickEvent event) {
				try { 
					userForm.commit();
					
					try {
						insertUserStatement.setString(1, user.getEmail());
						insertUserStatement.setString(2, user.getPassword());
						insertUserStatement.setString(3, user.getEmail());
						insertUserStatement.executeUpdate();
						System.out.println("committed");
						//dbConnection.close();
						navigator.navigateTo(MyLogin.viewId);
					} catch (SQLException e) {
						Notification.show("Unable to insert new user");
						System.err.println("commit failed due to database error");
						//okButton.setComponentError(new UserError("Unable to insert new user"));
					}
					
				} catch (CommitException e) {
					System.err.println("commit failed for unknown reason");
					e.printStackTrace();
				}
			}
		});
		
		password2Field.addValueChangeListener( new ValueChangeListener() {
			private static final long serialVersionUID = -4501346601822071684L;

			@Override
			public void valueChange(ValueChangeEvent event) {
				userForm.setValidationVisibility(true);
				password2Field.setValidationVisible(true);
				validateGlobal();
			}
		});
		userForm.setBuffered(true);
	}

	public void validateGlobal() {
		if (userForm.isValid() && password2Field.isValid()) {
			okButton.setEnabled(true);
		} else {
			okButton.setEnabled(false);
		}
	}
	
	@Override
	public void enter(ViewChangeEvent event) {
		user.setEmail(null);
		user.setPassword(null);
		user.setName(null);
		userForm.setItemDataSource(user);
		password2Field.setValue("");
		password2Field.setValidationVisible(false);
		userForm.setValidationVisibility(false);
	}
}
