package de.hdm_stuttgart.mi.tools;

import com.vaadin.data.Property.ValueChangeEvent;
import com.vaadin.data.Property.ValueChangeListener;
import com.vaadin.data.fieldgroup.BeanFieldGroup;
import com.vaadin.ui.AbstractField;
import com.vaadin.ui.Field;
import com.vaadin.ui.PasswordField;
import com.vaadin.ui.TextField;

/**
 * @author goik
 *
 * @param <T>
 */
public class MyBeanFieldGroup<T> extends BeanFieldGroup<T> {
	
	final ValidateGlobal validateGlobal;

	private static final long serialVersionUID = 757184578224921063L;

	/**
	 * @param beanType
	 * @param validateGlobal
	 */
	public MyBeanFieldGroup(final Class<T> beanType, final ValidateGlobal validateGlobal) {
		super(beanType);
		this.validateGlobal = validateGlobal;
	}

	/**
	 * @param beanType
	 */
	public MyBeanFieldGroup(final Class<T> beanType) {
		this(beanType, null);
	}

	class MyValueChangeListener implements ValueChangeListener {
		private static final long serialVersionUID = -877752164697692551L;
		final Field<?> [] upToCurrent = getFields().toArray(new Field<?>[0]);

		@Override
		public void valueChange(ValueChangeEvent event) {
			for (final Field<?> f: upToCurrent) {
				((AbstractField<?>) f).setValidationVisible(true);
			}
			if (null != validateGlobal) {
				validateGlobal.validateGlobal();
			}
		}
	}	
	
	@SuppressWarnings("rawtypes")
	@Override
	public <FT extends Field> FT buildAndBind(String caption, Object propertyId,
			Class<FT> fieldType) throws BindException {
		final FT ret = super.buildAndBind(caption, propertyId, fieldType);

		setNullRepresentation(ret);
		ret.addValueChangeListener(new MyValueChangeListener());
		return ret;
	}

	/**
	 * @param item
	 * @param initialValidationIsVisible
	 */
	public void setItemDataSource(T item, boolean initialValidationIsVisible) {
		super.setItemDataSource(item);
		if (!initialValidationIsVisible) {
			setValidationVisibility(false);
		}
	}

	@Override
	public Field<?> buildAndBind(final String caption, final Object propertyId)
			throws BindException {

		final Field<?> ret = super.buildAndBind(caption, propertyId);

		setNullRepresentation(ret);

		ret.addValueChangeListener(new MyValueChangeListener());
		return ret;
	}

	/**
	 * @param isVisible
	 */
	public void setValidationVisibility(final boolean isVisible) {
		for (final Field<?> f: getFields()) {
			((AbstractField<?>) f).setValidationVisible(isVisible);
		}
	}
	private void setNullRepresentation(final Field<?> field) {
		if (field instanceof TextField) {
			((TextField) field).setNullRepresentation("");
		} else if (field instanceof PasswordField) {
			((PasswordField) field).setNullRepresentation("");
		}
	}
}