package de.hdm_stuttgart.mi.sda2.usermanage.parse;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import de.hdm_stuttgart.mi.sda2.usermanage.rdbms.DatabaseObject;


public class User implements DatabaseObject {

	public static final Map<String, User> userByUid = new HashMap<String, User>();
	
	private int databaseId = 0;

	public final String uid;
	public final int uidNumber;
	public final int gidNumber;
	public final String cname;
	public final String homeDirectory;
	public final String shell;
	public final Group primaryGroup;
	public final Set<Group> secondaryGroups;


	public User(final String passwdLine, int row) 
			throws ParseError {

		int csvIndex = 0;
		final String [] components = passwdLine.split(":");

		// row 1
		uid = components[csvIndex++];

		csvIndex++; // We ignore the second CSV entry ("x") in row 2

		// row 3 
		try {
			uidNumber = Integer.parseInt(components[csvIndex++]);
		} catch (NumberFormatException e) {

			throw new ParseError("is not an integer (passwd)", row, csvIndex, components[csvIndex]);
		}

		// row 4
		try {
			gidNumber = Integer.parseInt(components[csvIndex++]);
		} catch (NumberFormatException e) {

			throw new ParseError("is not an integer (passwd)", row, csvIndex, components[csvIndex]);
		}

		// row 5
		cname = components[csvIndex++];

		// row 6
		homeDirectory = components[csvIndex++];

		// row 7
		shell= components[csvIndex++];

		// Group dependencies
		primaryGroup = Group.groupsByGidnumber.get(gidNumber);
		secondaryGroups = Group.secondaryGroupsByUid.get(uidNumber);
		userByUid.put(uid, this);
	}
	public int getDatabaseId() {
		return databaseId;
	}
	public void setDatabaseId(int databaseId) {
		this.databaseId = databaseId;
	}
}
