package de.hdm_stuttgart.mi.sda2.usermanage.rdbms;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.hdm_stuttgart.mi.sda2.usermanage.Cfg;
import de.hdm_stuttgart.mi.sda2.usermanage.parse.Group;
import de.hdm_stuttgart.mi.sda2.usermanage.parse.User;

public class RdbmsHandler {

	private static Logger log =  LogManager.getLogger(RdbmsHandler.class);

	private MyNamedQuery<GroupParam> insertGroupStmt;
	private MyNamedQuery<UserParam> insertUserStmt;
	private MyNamedQuery<UserGroupParam> insertUserGroupStmt;

	static {
		try {
			Class.forName(Cfg.get("RdbmsHandler.driverClassName"));
		} catch (ClassNotFoundException e) {
			System.err.println("Unable to register JDBC driver '"
					+ Cfg.get("RdbmsHandler.jdbcUrl") + "'");
			System.exit(1);
		}
	}

	Connection conn = null;

	public void connect() throws SQLException {
		if (null != conn) {
			conn.rollback();
			conn.close();
		}
		conn = DriverManager.getConnection(Cfg.get("RdbmsHandler.jdbcUrl"),
				Cfg.get("RdbmsHandler.user"), Cfg.get("RdbmsHandler.password"));
		conn.setAutoCommit(false);
		insertGroupStmt = new MyNamedQuery<GroupParam>(conn, GroupParam.gid, Statement.RETURN_GENERATED_KEYS);
		insertUserStmt = new MyNamedQuery<UserParam>(conn, UserParam.uid, Statement.RETURN_GENERATED_KEYS);
		insertUserGroupStmt = new MyNamedQuery<UserGroupParam>(conn, UserGroupParam.sysuser, Statement.RETURN_GENERATED_KEYS);
	}

	public void addGroup(final Group group) {

		try {
			insertGroupStmt.setString(GroupParam.gid, group.gid);
			insertGroupStmt.setInt(GroupParam.gidNumber, group.gidNumber);
			insertGroupStmt.getStatement().executeUpdate();
			addGeneratedId(insertGroupStmt.getStatement(), group);
		} catch (SQLException e) {
			log.fatal("Error inserting group '" + group.gid + "'");
			e.printStackTrace();
		}
	}

	public void addUser(final User user) {
		try {
			insertUserStmt.setString(UserParam.uid, user.uid);
			insertUserStmt.setInt(UserParam.uidNumber, user.uidNumber);
			insertUserStmt.setInt(UserParam.primaryGidNumber, user.gidNumber);
			insertUserStmt.setString(UserParam.cname, user.cname);
			insertUserStmt.setString(UserParam.homeDirectory, user.homeDirectory);
			insertUserStmt.setString(UserParam.shell, user.shell);

			insertUserStmt.getStatement().executeUpdate();
			addGeneratedId(insertUserStmt.getStatement(), user);
		} catch (SQLException e) {
			log.fatal("Error inserting group '" + user.uid + "'");
			e.printStackTrace();
		}
	}
	public void addUserGroup(final User user, final Group group) {
		try {
			insertUserGroupStmt.setInt(UserGroupParam.sysuser, user.getDatabaseId());
			insertUserGroupStmt.setInt(UserGroupParam.sysgroup, group.getDatabaseId());
			insertUserGroupStmt.getStatement().addBatch();
		} catch (SQLException e) {
			log.fatal("Error associating secondary group '" + group + "' to user '" + user.uid + "'");
			e.printStackTrace();
		}
		
	}
	public void commit() {
		try {
			insertUserGroupStmt.getStatement().executeBatch();
			conn.commit();
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	static void addGeneratedId(PreparedStatement stmt, final DatabaseObject dbo) 
			throws SQLException {
		final ResultSet rs = stmt.getGeneratedKeys();
		if (rs.next()) {
			final int ret = rs.getInt(1);
			log.info("Generated id=" + ret);
			dbo.setDatabaseId(ret);
		} else {
			log.fatal("No id found for '" + dbo + "', exiting!");
			System.exit(1);
		}
	}
}
