package de.hdm_stuttgart.mi.sda2.forum; 

import java.io.Serializable;

import com.vaadin.annotations.AutoGenerated;
import com.vaadin.data.Property.ValueChangeListener;
import com.vaadin.data.validator.RegexpValidator;
import com.vaadin.navigator.Navigator;
import com.vaadin.navigator.View;
import com.vaadin.navigator.ViewChangeListener.ViewChangeEvent;
import com.vaadin.ui.Alignment;
import com.vaadin.ui.Button;
import com.vaadin.ui.Button.ClickListener;
import com.vaadin.ui.CustomComponent;
import com.vaadin.ui.GridLayout;
import com.vaadin.ui.Label;
import com.vaadin.ui.Notification;
import com.vaadin.ui.PasswordField;
import com.vaadin.ui.TextField;
import com.vaadin.ui.VerticalLayout;

import de.hdm_stuttgart.mi.sda2.forum.domain.User;
import de.hdm_stuttgart.mi.sda2.forum.tools.Credential;

public class MyLogin extends CustomComponent implements View {

	/*- VaadinEditorProperties={"grid":"RegularGrid,20","showGrid":true,"snapToGrid":true,"snapToObject":true,"movingGuides":false,"snappingDistance":10} */

	@AutoGenerated
	private VerticalLayout mainLayout;
	@AutoGenerated
	private GridLayout buttonGrid;
	@AutoGenerated
	private Button loginButton;
	@AutoGenerated
	private Button registerButton;
	@AutoGenerated
	private GridLayout loginParameterGrid;
	@AutoGenerated
	private PasswordField passwordField;
	@AutoGenerated
	private Label passwordLabel;
	@AutoGenerated
	private TextField usernameField;
	@AutoGenerated
	private Label usernameLabel;
	@AutoGenerated
	private Label loginOfferLabel;
	// Manual modifications
	private static final long serialVersionUID = 5701668846027080460L;
	public final static String viewId = MyLogin.class.getCanonicalName();

	private final Navigator navigator;
	/**
	 * The constructor should first build the main layout, set the
	 * composition root and then do any custom initialization.
	 *
	 * The constructor will not be automatically regenerated by the
	 * visual editor.
	 */
	public MyLogin(final Navigator navigator) {
		this.navigator = navigator;
		buildMainLayout();
		setCompositionRoot(mainLayout);
		
		// Login requires a valid username
		loginButton.setEnabled(false);
		
		// Entering a valid username should enable our login button
		usernameField.setImmediate(true);
		
		usernameField.addValidator(new RegexpValidator("[@a-zA-Z0-9]+",
				"Username must contain only letters and digits e.g. 'trustix'"));
		
		usernameField.addValueChangeListener((ValueChangeListener & Serializable) event -> 
			loginButton.setEnabled(usernameField.isValid()));
		
		registerButton.addClickListener((ClickListener & Serializable) event ->
			navigator.navigateTo(Register.viewId));
		
		loginButton.addClickListener((ClickListener & Serializable) event -> 
		    checkLogin());
	}

	private void checkLogin() {
		final User user = Credential.check(usernameField.getValue(), passwordField.getValue());
		if (null == user) {
			Notification.show("Username and password do not match");
		} else {
			navigator.navigateTo(Main.viewId);
		}
	}
	
	
	@Override
	public void enter(ViewChangeEvent event) {
		usernameField.setValue("");
		passwordField.setValue("");
	}

	@AutoGenerated
	private VerticalLayout buildMainLayout() {
		// common part: create layout
		mainLayout = new VerticalLayout();
		mainLayout.setImmediate(false);
		mainLayout.setWidth("400px");
		mainLayout.setHeight("400px");
		mainLayout.setMargin(true);
		mainLayout.setSpacing(true);
		
		// top-level component properties
		setWidth("400px");
		setHeight("400px");
		
		// loginOfferLabel
		loginOfferLabel = new Label();
		loginOfferLabel.setImmediate(false);
		loginOfferLabel.setWidth("-1px");
		loginOfferLabel.setHeight("-1px");
		loginOfferLabel.setValue("Please log in");
		mainLayout.addComponent(loginOfferLabel);
		mainLayout.setComponentAlignment(loginOfferLabel, new Alignment(33));
		
		// loginParameterGrid
		loginParameterGrid = buildLoginParameterGrid();
		mainLayout.addComponent(loginParameterGrid);
		mainLayout.setComponentAlignment(loginParameterGrid, new Alignment(48));
		
		// buttonGrid
		buttonGrid = buildButtonGrid();
		mainLayout.addComponent(buttonGrid);
		mainLayout.setComponentAlignment(buttonGrid, new Alignment(24));
		
		return mainLayout;
	}


	@AutoGenerated
	private GridLayout buildLoginParameterGrid() {
		// common part: create layout
		loginParameterGrid = new GridLayout();
		loginParameterGrid.setImmediate(false);
		loginParameterGrid.setWidth("-1px");
		loginParameterGrid.setHeight("-1px");
		loginParameterGrid.setMargin(true);
		loginParameterGrid.setSpacing(true);
		loginParameterGrid.setColumns(2);
		loginParameterGrid.setRows(2);
		
		// usernameLabel
		usernameLabel = new Label();
		usernameLabel.setImmediate(false);
		usernameLabel.setWidth("-1px");
		usernameLabel.setHeight("-1px");
		usernameLabel.setValue("Username:");
		loginParameterGrid.addComponent(usernameLabel, 0, 0);
		
		// usernameField
		usernameField = new TextField();
		usernameField.setImmediate(false);
		usernameField.setWidth("-1px");
		usernameField.setHeight("-1px");
		loginParameterGrid.addComponent(usernameField, 1, 0);
		
		// passwordLabel
		passwordLabel = new Label();
		passwordLabel.setImmediate(false);
		passwordLabel.setWidth("-1px");
		passwordLabel.setHeight("-1px");
		passwordLabel.setValue("Password:");
		loginParameterGrid.addComponent(passwordLabel, 0, 1);
		
		// passwordField
		passwordField = new PasswordField();
		passwordField.setImmediate(false);
		passwordField.setWidth("-1px");
		passwordField.setHeight("-1px");
		loginParameterGrid.addComponent(passwordField, 1, 1);
		
		return loginParameterGrid;
	}


	@AutoGenerated
	private GridLayout buildButtonGrid() {
		// common part: create layout
		buttonGrid = new GridLayout();
		buttonGrid.setImmediate(false);
		buttonGrid.setWidth("-1px");
		buttonGrid.setHeight("-1px");
		buttonGrid.setMargin(true);
		buttonGrid.setSpacing(true);
		buttonGrid.setColumns(3);
		
		// registerButton
		registerButton = new Button();
		registerButton.setCaption("Register");
		registerButton.setImmediate(true);
		registerButton.setWidth("-1px");
		registerButton.setHeight("-1px");
		buttonGrid.addComponent(registerButton, 0, 0);
		
		// loginButton
		loginButton = new Button();
		loginButton.setCaption("Log in");
		loginButton.setImmediate(true);
		loginButton.setWidth("-1px");
		loginButton.setHeight("-1px");
		buttonGrid.addComponent(loginButton, 2, 0);
		
		return buttonGrid;
	}

}
