package de.hdm_stuttgart.mi.sda2.forum;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;

import org.junit.Test;

import de.hdm_stuttgart.mi.sda2.forum.domain.Forum;
import de.hdm_stuttgart.mi.sda2.forum.domain.User;
import de.hdm_stuttgart.mi.sda2.forum.tools.HashProvider;

@SuppressWarnings("javadoc")
public class TestDomain {

	static final EntityManagerFactory emf;

	static final EntityManager em;

	static {
		final Map<String, Object> overrideAndAddCfg = new HashMap<String, Object>();
		overrideAndAddCfg.put("eclipselink.ddl-generation", "drop-and-create-tables");
		emf = Persistence.createEntityManagerFactory("de.hdm_stuttgart.mi.sda2.forum", overrideAndAddCfg);
		em = emf.createEntityManager(overrideAndAddCfg);
	}
	
	@Test
	public void test() {
		
		//em.setProperty("eclipselink.ddl-generation", "drop-and-create-tables");
		
		try {
			
			em.getTransaction().begin();
			persistUser ("Joe Flix",    "joe@due.com",         "q");
			persistUser("Tim Burns",    "burns@somefolks.org", "q");
			persistUser("Sarah Beans",  "beans@myuni.edu",     "q");
			persistUser("Peter Sounds", "q@q",                 "q");
			persistUser("Sam Drews",    "sdew@bb.cc.org",      "q");
			persistUser("Kim Banff",    "kb@banff.org",        "q");
			
			persistForum("Java delepopment problems");
			em.getTransaction().commit();
			
		} catch (Exception e) {
	        if (e instanceof ConstraintViolationException) {
	            ConstraintViolationException cve = (ConstraintViolationException) e;
	            for (Iterator<ConstraintViolation<?>> it = cve.getConstraintViolations().iterator(); it.hasNext();) {
	                ConstraintViolation<? extends Object> v = it.next();
	                System.err.println(v);
	                System.err.println("==>>"+v.getMessage());
	            }
	        } else {
				e.printStackTrace();
	        }
	        throw e;
		}
	}

	private static User persistUser(final String name, final String email, final String clearTextPassword) {
		System.out.print("Starting transaction (user = '" + name + "') ... ");
		final User user = new User(name, email, HashProvider.getSaltedHash(clearTextPassword.toCharArray()));
		em.persist(user);
		System.out.println("done!");
		return user;
	}

	private static Forum persistForum(final String title) {
		System.out.print("Starting transaction (forum = '" + title + "') ... ");
		final Forum forum = new Forum(title);
		em.persist(forum);
		System.out.println("done!");
		return forum;
	}
}
