package de.hdm_stuttgart.mi.sd1.aufgabe2;

/**
 * <p>Verzinsung von Guthaben oder Schulden. Es gelten folgende Regeln:</p>
 *
 * <p>Einmal pro Jahr kann über eine der beiden Methoden  {@link #neuesJahr(double, double, double)} oder {@link #neuesJahr(double)}
 *   gemäß Vorzeichen des Betrags eingezahlt oder abgehoben werden.</p>
 *
 * <p>Gleichzeitig können zu diesem Zeitpunkt die Zinssätze für Guthaben und Schulden (Soll) für das Folgejahr angepasst werden.</p>
 *
 */
public class Konto {

  private int jahr = 0;

  private double kontostand = 0,
    guthabenZins, sollZins;

  private final StringBuffer journal = new StringBuffer();

  private void verzinsung() {
    if (0 < kontostand) {
      kontostand *= (1 + guthabenZins / 100);
    } else {
      kontostand *= (1 + sollZins / 100);
    }
    jahr++;
  }

  private void journalEintrag() {
    journal.append("Jahr ").append(jahr).append(", Kontostand=").append(Math.round(kontostand));
    if (0 <= kontostand) {
      journal.append(", verzinsung=").append(guthabenZins);
    } else {
      journal.append(", verzinsung=-").append(sollZins);
    }
    journal.append("%\n");
  }

  /**
   * Erstelle neues Konto.
   *
   * @param kontostand Initialer Kontostand.
   * @param guthabenZins Initialer Guthabenzins.
   * @param sollZins Initialer Schuldzins.
   */
  public Konto(double kontostand, double guthabenZins, double sollZins) {
    this.kontostand = kontostand;
    this.guthabenZins = guthabenZins;
    this.sollZins = sollZins;
    journalEintrag();
  }

  /**
   * <p>Diese oder alternativ die Methode {@link #neuesJahr(double)} wird einmal
   * jährlich ausgeführt.</p>
   *
   * <p>Die Methode bewirkt folgende Änderungen:</p>
   *
   * <ul>
   *   <li>Durchführung der Verzinsung für das zurückliegende Jahr zu aktuellen Guthaben- bzw. Schuldzinsen.</li>
   *   <li>Einzahlung für das jeweils nächste Jahr</li>
   *   <li>Anpassung von Guthaben- und Schuldzins für das kommende Verzinsungsjahr.</li>
   * </ul>
   *
   * @param betrag Je nach Vorzeichen neu hinzugefügter oder abgebuchter Betrag.
   * @param guthabenZins Der für das jeweils kommende Jahr geltende prozentuale Guthabenzins für nichtnegative Kontostände.
   * @param sollZins Der für das jeweils kommende Jahr geltende prozentuale Guthabenzins für negative Kontostände.
   *
   * <p style="color: red;font-weight: bold;">Tip: Im Hinblick auf das Journal in {@link #getJournal()} können Sie alle
   * benötigten Informationen in einer {@link StringBuffer} Instanzvariablen sammeln. Sinngemäß gilt dies auch für den jeweils
   * aktuellen Kontostand.</p>
   */
  public void neuesJahr(double betrag, double guthabenZins, double sollZins) {
    verzinsung();
    this.guthabenZins= guthabenZins;
    this.sollZins = sollZins;
    kontostand += betrag;
    journalEintrag();
  }

  /**
   * Wie {@link #neuesJahr(double, double, double)} allerdings ohne Änderung von Guthaben- und Sollzins für
   * das kommende Verzinsungsjahr.
   *
   * @param betrag Je nach Vorzeichen neu hinzugefügter oder abgebuchter Betrag.
   */
  public void neuesJahr(double betrag) {
    verzinsung();
    kontostand += betrag;
    journalEintrag();
  }

  /**
   * @return Der aktuelle Kontostand.
   */
  public double getKontostand() {
    return kontostand;
  }

  /**
   * <p>Die Entwicklung der Kontostände kaufmännisch auf ganzzahlige Werte gerundet.
   * Für das Beispiel {@link DummyZinsen#main(String[])} ergibt sich:</p>
   *
   * <pre>Jahr 0, Kontostand=100, verzinsung=1.0%
   *Jahr 1, Kontostand=201, verzinsung=1.0%
   *Jahr 2, Kontostand=403, verzinsung=2.0%
   *Jahr 3, Kontostand=-89, verzinsung=-8.0%</pre>
   *
   * @return Ausdruckbarer Verlauf der Kontoentwicklung
   *
   * <ul>
   *   <li><p style="color: red;font-weight: bold;">Tip: Im Hinblick auf das Journal in können Sie alle
   * benötigten Informationen in einer {@link StringBuffer} Instanzvariablen sammeln. Sinngemäß gilt dies
   * auch für den jeweils aktuellen Kontostand. Beachten Sie die Methode {@link Math#round(double)}</p></li>
   *
   * <li><p style="color: red;font-weight: bold;">Tip: Ein Hang zum Erbsenzählen erleichtert die Lösung.
   * Bereits ein falsches Leer- oder fehlendes Newline Zeichen (auch am Ende!)
   * lässt die zugehörigen Tests scheitern. Beachten Sie allfällige Junit Fehlermeldungen sehr genau.</p></li>
   *
   * </ul>
   *
   */
  public String getJournal() {
    return journal.toString();
  }
}
