package de.hdm_stuttgart.mi.exam.unitmarking;

import org.junit.Rule;
import org.junit.rules.DisableOnDebug;
import org.junit.rules.TestRule;
import org.junit.rules.Timeout;

/**
 * This base class presets the following test properties:
 * 
 * <ul>
 *   <li>Limit test method execution to 1s (default).</li>
 * </ul>
 *
 */
public abstract class ExaminationTestDefaults {
   /**
    * <ul>
    * 
    *   <li>Each individual test requiring more than 1s of execution
    *       time will be aborted with Status »failed«. This may be due
    *       to e.g. infinite loop or similar problems.</li>
    *        
    *   <li>Test abortion will not happen in debugging mode.</li>
    * </ul>
    */
   @Rule
   public final TestRule timeout;
   
   /**
    * Setting test abortion timeout to 1000 milliseconds.
    */
   public ExaminationTestDefaults() {
      timeout = new DisableOnDebug(Timeout.millis(1000));
   }

   /**
    * Specify test method abortion timeout.
    * 
    * @param timeout Setting test abortion timeout to the given value
    *        in milliseconds.
    */
   public ExaminationTestDefaults(final int timeout) {
      this.timeout = new DisableOnDebug(Timeout.millis(timeout));
   }
}
