<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="2.0"
    xmlns:xlink="http://www.w3.org/1999/xlink"  xmlns:d="http://docbook.org/ns/docbook">
    
    <xsl:import href="/usr/share/xmlmind/addon/config/docbook5/xsl/html/docbook.xsl" />
        
    <xsl:template match="/">
        <xsl:apply-templates select="//d:chapter[d:annotation[@role='slide']] | //d:section[d:annotation[@role='slide']] | 
            //d:appendix[d:annotation[@role='slide']]" mode='entry'/>        
    </xsl:template>
    
    <xsl:template match="d:chapter | d:section | d:appendix" mode='entry'>
        <xsl:result-document method="xhtml" encoding="utf-8" href="Slides/{@xml:id}.html" indent="no">
            <html lang="en">
                <head>
                    <title>
                        <xsl:value-of select="d:title"/>
                    </title>
                    <link href="mi.css" rel="stylesheet" />
                    <link href="reveal.js/css/reveal.css" rel="stylesheet" />
                    <link href="reveal.js/css/theme/solarized.css" id="theme" rel="stylesheet" />
                    
                    <!-- Code syntax highlighting -->
                    <link href="reveal.js/lib/css/zenburn.css" rel="stylesheet" />
                    
                </head>
                
                <body>
                    <!-- Wrap the entire slide show in a div using the "reveal" class. -->
                    <div class="reveal">
                        <!-- Wrap all slides in a single "slides" class -->
                        <div class="slides">
                            <section id="{@xml:id}">
                                <xsl:apply-templates select="*[not(self::d:section)]"/>
                            </section>
                            <xsl:apply-templates select="d:section">
                                <xsl:with-param name="titleset" select="d:title"/>
                            </xsl:apply-templates>
                        </div>
                    </div>
                    
                    <script src="reveal.js/lib/js/head.min.js"></script>
                    <script src="reveal.js/js/reveal.js"></script>
                    <script>/*<![CDATA[*/
			
			Reveal.initialize({
			controls: true,
			progress: true,
			history: true,
			center: true,
			
			transition: 'slide', // none/fade/slide/convex/concave/zoom
			
			math: {
		        mathjax: 'https://prog.mi.hdm-stuttgart.de/mathjax/MathJax.js',
		        config: 'TeX-AMS-MML_HTMLorMML',  // See http://docs.mathjax.org/en/latest/config-files.html
		    },

            // Optional reveal.js plugins
			dependencies: [
			{ src: 'reveal.js/lib/js/classList.js', condition: function() { return !document.body.classList; } },
			{ src: 'reveal.js/plugin/markdown/marked.js', condition: function() { return !!document.querySelector( '[data-markdown]' ); } },
			{ src: 'reveal.js/plugin/markdown/markdown.js', condition: function() { return !!document.querySelector( '[data-markdown]' ); } },
			{ src: 'reveal.js/plugin/highlight/highlight.js', async: true, callback: function() { hljs.initHighlightingOnLoad(); } },
			{ src: 'reveal.js/plugin/zoom-js/zoom.js', async: true },
			{ src: 'reveal.js/plugin/notes/notes.js', async: true },
            { src: 'reveal.js/plugin/math/math.js', async: true }
			]
			});
			/*]]>*/</script>
                </body>
            </html>
        </xsl:result-document>
    </xsl:template>
    
    <xsl:template match="d:section">
        <xsl:param name="titleset"/>
        <xsl:variable name="extendedTitleset" select="$titleset|d:title"/>
        <section id="{@xml:id}">
            <xsl:call-template name="titleNavigation">
                <xsl:with-param name="titleset" select="$extendedTitleset"/>
            </xsl:call-template>
            <xsl:apply-templates select="*[not(self::d:section | self::d:title)]"/>
        </section>
        <xsl:apply-templates select="d:section">
            <xsl:with-param name="titleset" select="$extendedTitleset"/>
        </xsl:apply-templates>
    </xsl:template>
    
    <xsl:template name="titleNavigation">
        <xsl:param name="titleset"/>
        <h4>
            <xsl:for-each select="$titleset[1]">
                <a href="#{parent::*/@xml:id}">
                    <!-- <xsl:value-of select="."/> -->
                    <xsl:text>Top</xsl:text><!-- Overall title is likely too long -->
                </a>
            </xsl:for-each>

            <xsl:for-each select="$titleset[1 &lt; position() and position() &lt; last()]">
                <xsl:text> &gt; </xsl:text>
                <a href="#{parent::*/@xml:id}">
                    <xsl:value-of select="."/>
                </a>
            </xsl:for-each>

            <xsl:for-each select="$titleset[1 &lt; position() and position() = last()]">
                <xsl:text> &gt; </xsl:text>
                <xsl:value-of select="."/>
            </xsl:for-each>
        </h4>
    </xsl:template>
    
    <xsl:template match="d:title">
        <h4>
            <xsl:value-of select="."/>
        </h4>
    </xsl:template>
    
</xsl:stylesheet>
