package de.hdm_stuttgart.mi.bb;

import com.mongodb.MongoClient;
import com.mongodb.MongoClientURI;
import com.mongodb.client.MongoCollection;
import com.mongodb.client.MongoDatabase;

import static com.mongodb.client.model.Filters.eq;

import static com.mongodb.client.model.Updates.combine;
import static com.mongodb.client.model.Updates.set;

import org.bson.Document;

class ModifyNodebbTopic {
    ModifyNodebbTopic(final String uri, final String databaseName) {
        this.uri = uri;
        this.database = new MongoClient(new MongoClientURI(uri)).getDatabase(databaseName);
        objectCollection = database.getCollection("objects");
    }

    private final MongoCollection<Document> objectCollection;

    /**
     * Check for the existence of the hdmId attribute. If present, return
     * the topic's tid value, else return a negative value.
     *
     * @param hdmId The hdmId value to be searched for.
     * @return If a topic carrying the hdmId value exists, return its tid, else return a negative value.
     */
     int hdmIdExists(final String hdmId) {

        final Document res = objectCollection.find(eq("hdmId", hdmId)).first();
        if (null == res) {
            return -1;
        } else {
            return res.getInteger("tid");
        }
    }

    /**
     * Search for a topic with given tid value. If existent the hdmId value will be added.
     * @param tidKey A topic's _key value.
     * @param hdmId Value will be added as "hdmId" attribute unless already being defined.
     */
    void addHdmId(final int tidKey, final String hdmId) {

        final String searchKey = "topic:" + tidKey;
        final Document topic = objectCollection.find(eq("_key", searchKey)).first();

        if (null != topic) {

            final String currentHdmId = topic.getString("hdmId");

            if (null == currentHdmId) {
                objectCollection.updateOne(
                        eq("_key", searchKey),
                        combine(set("hdmId", hdmId)));
                System.out.println("Setting new hdmId value '" + hdmId + "'");

            } else if (currentHdmId.equals(hdmId)) {
                System.out.println("hdmId already set to value '" + currentHdmId + "', no action");
            } else {
                System.out.println("current hdmId '" + currentHdmId + "' differs from '" + hdmId + "', no action");
            }
        }
    }

    private final String uri;
    private final MongoDatabase database;
}
