package dom.xpath;

import java.io.IOException;

import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.JDOMException;
import org.jdom2.Namespace;
import org.jdom2.filter.ElementFilter;
import org.jdom2.input.SAXBuilder;
import org.jdom2.xpath.XPathExpression;
import org.jdom2.xpath.XPathFactory;

import dom.MySaxErrorHandler;

/** 
 * How to use XPath expressions to search for &lt;img&gt;
 *  elements in XHTML document instances.
 *  
 */
public class DomXpath {
   private final SAXBuilder builder = new SAXBuilder();

   /**
   */
   public DomXpath() {
      builder.setErrorHandler(new MySaxErrorHandler(System.err));
   }
   /** 
    * Read an XHTML document and search for images. The image filenames will be
    * written to standard out.
    * @param xhtmlFilename The XHTML's filename to be examined.
    * @throws JDOMException General parsing fault
    * @throws IOException file access problem
    */
   public void process(final String xhtmlFilename) throws JDOMException, IOException {

      final Document htmlInput = builder.build(xhtmlFilename);
      
      final Namespace htmlNamespace = // This will allow for XPath expressions like xhtml:img
            Namespace.getNamespace("h", "http://www.w3.org/1999/xhtml");
      
      final XPathExpression<Element> searchImg = XPathFactory.instance().compile (
            "//h:img",
            new ElementFilter(), null, htmlNamespace);
      
      searchImg.evaluate(htmlInput).
         stream().
         map(img -> img.getAttribute("src").getValue()).
         forEach(System.out::println);
   }
}