package sax.count;

import java.util.SortedSet;
import java.util.TreeSet;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/** Reading attributes from element events */
public class ListTagNamesHandler extends DefaultHandler {

  // A SortedSet by definition does not contain any duplicates.
  private SortedSet<String> elementNames = new TreeSet<>();

  @Override
  public void startDocument() throws SAXException {
    elementNames.clear(); // May contain elements from a previous run.
  }

  public void startElement(String namespaceUri, String localName,
      String rawName, Attributes attrs) {
    // In case the current element name has already been inserted
    // this method call will be silently ignored.
    elementNames.add(rawName);
  }

  /**
   * @return A sorted list of element names of he currently processed XML
   *         document without duplicates.
   */
  public String[] getTagNames() {
    return elementNames.toArray(new String[0]);
  }
}