package sax.mixed;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

public class MixedCountHandler extends DefaultHandler {
   
   enum EventType {openElement, closeElement, character, idle};
   EventType lastEvent = EventType.idle;

   private int mixedElementCount = 0;

   @Override
   public void startElement(String uri, String localName, String qName,
         Attributes attributes) throws SAXException {
      if (lastEvent == EventType.character) {
         mixedElementCount++;
      }
      lastEvent = EventType.openElement;
   }

   @Override
   public void endElement(String uri, String localName, String qName)
         throws SAXException {
      lastEvent = EventType.closeElement;
   }

   @Override
   public void characters(char[] ch, int start, int length) throws SAXException {
      final String content = new String(ch, start, length);
      
      // We ignore whitespace completely
      if (0 < content.trim().length()) {
         if (lastEvent == EventType.closeElement) {
            mixedElementCount++;
         }
         lastEvent = EventType.character;
      }
   }
   
   public int getMixedElementCount() {
      return mixedElementCount;
   }
}