package sda.jdbc.intro.auth;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;


/**
 * This class helps to keep track of index place holder values in
 * PreparedStatement objects.
 * 
 * If the SQL code gets modified a corresponding setParameter(...) Method
 * in a derived concrete class has to be adjusted as well.
 *
 * @author goik
 *
 */
public abstract class PreparedStatementHandler {

   final protected PreparedStatement pStmt;
   public PreparedStatementHandler(final Connection conn, final String sql) 
         throws SQLException {
      pStmt = conn.prepareStatement(sql);
   }
   public final ResultSet executeQuery() throws SQLException {
      return pStmt.executeQuery();
   }
   public final int executeUpdate() throws SQLException {
      return pStmt.executeUpdate();
   }
}
