package sda.jdbc.intro.v1.prepare;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.sql.SQLException;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

public class InsertPerson extends JFrame {
   
   private static final long serialVersionUID = 6815975741605247675L;
   
   final PersistenceHandler persistenceHandler = new PersistenceHandler();
   
   final JTextField nameField = new JTextField(15),
                   emailField = new JTextField(20);

   final JButton toggleConnectButton = new JButton(),
                        insertButton = new JButton("Insert");

   final JPanel databaseFieldPanel = new JPanel();

   private void setDatabaseConnectionState(final boolean state) {
      if (state) {
         toggleConnectButton.setText("Disconnect");
      } else {
         toggleConnectButton.setText("Connect");
      }
      for (final Component c: databaseFieldPanel.getComponents()){
         c.setEnabled(state);
      }
   }

   public static void main(String[] args) throws SQLException {
      InsertPerson app = new InsertPerson();
      app.setVisible(true);
   }
   
   public InsertPerson (){
      super ("SQL injection killer");
      
      setSize(500, 500);
      
      addWindowListener(new WindowAdapter() {
         // In case a user closes our application window while still being connected
         // we have to close the database connection.
         @Override
         public void windowClosing(WindowEvent e) {
            super.windowClosing(e);
            if (persistenceHandler.isConnected() && !persistenceHandler.disconnect()) {
               System.exit(1);
            } else {
               System.exit(0);
            }
         }
      });
      Box top = Box.createHorizontalBox();
      add(top, BorderLayout.NORTH);
      top.add(toggleConnectButton);
      
      toggleConnectButton.addActionListener(new ActionListener() {
         @Override
         public void actionPerformed(ActionEvent e) {
            if (persistenceHandler.isConnected()) {
               if (persistenceHandler.disconnect()){
                  setDatabaseConnectionState(false);
               } else {
                  JOptionPane.showMessageDialog(null, persistenceHandler.getErrorMessage());
               }
            } else {
               if (persistenceHandler.connect()){
                  setDatabaseConnectionState(true);
               } else {
                  JOptionPane.showMessageDialog(null, persistenceHandler.getErrorMessage());
               }
            }
         }
      });
      
      databaseFieldPanel.setLayout(new GridLayout(0, 2));
      add(databaseFieldPanel);

      databaseFieldPanel.add(new JLabel("Name:"));
      databaseFieldPanel.add(nameField);
      
      databaseFieldPanel.add(new JLabel("E-mail:"));
      databaseFieldPanel.add(emailField);
      
      insertButton.addActionListener(new ActionListener() {
        @Override
        public void actionPerformed(ActionEvent e) {
           if (persistenceHandler.add(nameField.getText(), emailField.getText())) {
              nameField.setText("");
              emailField.setText("");
              JOptionPane.showMessageDialog(null, "Succesfully inserted dataset");
           } else {
              displayWrappedMessage("Error: " + persistenceHandler.getErrorMessage());
           }
        }
      });
      databaseFieldPanel.add(Box.createGlue());
      databaseFieldPanel.add(insertButton);   
      setDatabaseConnectionState(false);
      pack();
   }
   private void displayWrappedMessage(final String msg) {
      final JTextArea errorArea = new JTextArea();
      errorArea.setText(msg);
      errorArea.setRows(4);
      errorArea.setColumns(40);
      errorArea.setEditable(false);
      errorArea.setLineWrap(true);
      JOptionPane.showMessageDialog(null, new JScrollPane(errorArea));
   }
}
